// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "EntitySwordAttack.hpp"
#include "prefabs/SwordHitbox.hpp"

using namespace Dawn;

EntitySwordAttack::EntitySwordAttack(SceneItem* item) : EntityAttackBase(item) {
}

void EntitySwordAttack::onStart() {
  EntityAttackBase::onStart();
  
  this->hurtboxItem = SwordHitbox::create(this->getScene());
  this->hurtboxItem->transform.setParent(this->transform);

  useEffect([&]{
    switch(this->state) {
      case ENTITY_ATTACK_STATE_ACTIVE: {
        auto myFaction = item->getComponent<EntityFaction>();
        if(myFaction != nullptr) this->hurtboxItem->hazard->faction = myFaction->faction;
        this->hurtboxItem->transform.setLocalPosition(this->attackOffset);
        break;
      }
      default: {
        this->hurtboxItem->transform.setLocalPosition(glm::vec3(99999999,0,0));
        break;
      }
    }
  }, this->state)();
}

void EntitySwordAttack::onDispose() {
  EntityAttackBase::onDispose();
  this->hurtboxItem->destroy();
}

float_t EntitySwordAttack::getAttackRampUpDuration() {
  return 0.1f;
}

float_t EntitySwordAttack::getAttackActiveDuration() {
  return 0.2f;
}

float_t EntitySwordAttack::getAttackRampDownDuration() {
  return 0.1f;
}

float_t EntitySwordAttack::getAttackCooldownDuration() {
  return 0.0f;
}

bool_t EntitySwordAttack::isInterruptable() {
  return false;
}

bool_t EntitySwordAttack::canAttack() {
  return true;
}